searchData={"content_type":"text/markdown","items":[{"doc":"Elixir Parser Combinators\nAuthor: Matt Mower  \nVersion: 0.9.1","ref":"Ergo.html","title":"Ergo","type":"module"},{"doc":"If you decide to use Ergo and you want help please come find me in the [Elixir Discord](https://discord.gg/elixir) (where I am __sandbags__). Regardless, I would love to hear from you about problems and or suggestions for improvement.\n\n# What is Ergo?\n\nErgo is an Elixir language parser combinator library. The name 'ergo' means 'therefore' means 'for that reason' which seemed appropriate for a parser.\n\nAlso note that this is the second attempt I've made at building such a library. The first, Epic, was not completed and, on reflection, not good maintanable code. I did, however, learn a great deal in building it and that learning has put Ergo on a much firmer footing.\n\n# Using Ergo","ref":"Ergo.html#module-getting-help","title":"Getting Help - Ergo","type":"module"},{"doc":"Add Ergo to your mix.exs file:\n\n    {:ergo, \"~> 0.9\"}\n\nThen run\n\n    mix deps.get","ref":"Ergo.html#module-installing","title":"Installing - Ergo","type":"module"},{"doc":"See the [guide](basic_parser.md) to parsing.","ref":"Ergo.html#module-a-basic-parser","title":"A Basic Parser - Ergo","type":"module"},{"doc":"As with most parser combinator libraries, Ergo parsers are anonymous functions parameterized by inputs. However Ergo parser functions are wrapped in an `Ergo.Parser` struct that holds the parser function and can contain additional descriptive metadata (by default a description of the parser behaviour).","ref":"Ergo.html#module-what-is-a-parser","title":"What is a parser? - Ergo","type":"module"},{"doc":"Ergo parsers operate on a `Context` struct that gets passed from parser to parser. We can create the context directly:\n\n```elixir\nalias Ergo.Context\nctx = Context.new(\"string to be parsed\")\n```\n\nAlthough you will rarely need to do so as there is a helper function `Ergo.parse` that will do that for you:\n\n```elixir\nErgo.parse(parser, \"string to be parsed\")\n```\n\nErgo can send debugging information to the Elixir Logger which can be helpful in figuring out why a parser is not working as expected. E.g.\n\n```elixir\nErgo.parse(parser, \"string to be parsed\", debug: true)\n```\n\nThe `status` field of the context will always be either `:ok` or a tuple of `{:error, :error_atom}`.\n\nThe parser can build up a datastructure to return by modifying the `ast` value of the context. The `sequence` parser, for example, creates a list of all matching parsers, e.g.\n\n    Ergo.parse(sequence([uint(), ignore(char(?,)), uint()]), \"1,3\")\n    => %Context{status: :ok, ast: [1, 3]}","ref":"Ergo.html#module-the-context","title":"The Context - Ergo","type":"module"},{"doc":"At the heart of Ergo is the `Context` record. All Ergo parser functions take and return a `Context`. A `Context` has the following fields:\n\n    status\n    message\n    input\n    index\n    line\n    col\n    char\n    ast","ref":"Ergo.html#module-the-context-in-detail","title":"The Context in detail - Ergo","type":"module"},{"doc":"Every parser sets its status to either `:ok` or a tuple whose first element is the atom `:error` and whose second element is a list of `{code, message}` tuples\ndescribing the error in more detail for example `{:error, [{:unexpected_char, \"Expected 'A' but found '1'}]}`.","ref":"Ergo.html#module-status","title":"status - Ergo","type":"module"},{"doc":"A binary containing the input remaining to be matched.","ref":"Ergo.html#module-input","title":"input - Ergo","type":"module"},{"doc":"The index of the next character to be matched in the input.","ref":"Ergo.html#module-index","title":"index - Ergo","type":"module"},{"doc":"The current line of the input the parser has reached.","ref":"Ergo.html#module-line","title":"line - Ergo","type":"module"},{"doc":"The current column of the input line that the parser has reached.","ref":"Ergo.html#module-col","title":"col - Ergo","type":"module"},{"doc":"The data structure that the parser constructs from its input.\n\n# Parsers\n\nErgo comes with a set of basic parsers with which you can assemble your own, more complex, parsers. These are in the form of terminal parsers, combinator parsers, and numeric parsers.","ref":"Ergo.html#module-ast","title":"ast - Ergo","type":"module"},{"doc":"The terminal parsers are not parameterized by another parser and, in general, are low-level parsers for matching specified characters or literal sequences.","ref":"Ergo.html#module-terminal-parsers","title":"Terminal Parsers - Ergo","type":"module"},{"doc":"The `eoi` parser returns `:ok` if the input is empty otherwise returns `{:error, unexpected_eoi}`.","ref":"Ergo.html#module-eoi","title":"eoi() - Ergo","type":"module"},{"doc":"The `char` parser when given a single character code matches that character. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the character `c`. Otherwise it returns `status: {:error, :unexpected_char}`\n\nExample: char(?a) matches the character 'a'","ref":"Ergo.html#module-char-c","title":"char(?c) - Ergo","type":"module"},{"doc":"The `char` parser when given a range of characters. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the matched character. Otherwise it returns `status: {:error, :unexpected_char}`\n\nExample: char(?A..?Z) matches any uppercase letter","ref":"Ergo.html#module-char-l-h","title":"char(?l..?h) - Ergo","type":"module"},{"doc":"The `char` parser when given a negative character matches any character except the specified character and return `:ok` with the `char` and `ast` parameters set to the matched character. Otherwise it returns `status: {:error, :unexpected_char}`\n\nExample: char(-?,) matches any character except a comma","ref":"Ergo.html#module-char-c","title":"char(-?c) - Ergo","type":"module"},{"doc":"The `char` parser when given a list matches a character according to the specifications in the list which can be either ?c, -?c, or [?l..?h]. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the matched character. Otherwise it returns `status: {:error, :unexpected_char}`\n\nExample: char([a..z, 5]) matches any lower case letter or the number 5","ref":"Ergo.html#module-char","title":"char([...]) - Ergo","type":"module"},{"doc":"The `digit` parser matches a character in the range `[0..9]` from the input. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the character code of the digit that has been matched. Otherwise it returns `status: {:error, :unexpected_char}`","ref":"Ergo.html#module-digit","title":"digit - Ergo","type":"module"},{"doc":"The `alpha` parser matches a character in the range `[a..z, A..Z]` from the input. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the character code of the alpha character that has been matched. Otherwise it returns `status: {:error, :unexpected_char}`","ref":"Ergo.html#module-alpha","title":"alpha - Ergo","type":"module"},{"doc":"The `wc` parser matches a word character from the input (it is equivalent to the \\w regular expression). If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the character code of the alpha character that has been matched. Otherwise it returns `status: {:error, :unexpected_char}`.","ref":"Ergo.html#module-wc","title":"wc - Ergo","type":"module"},{"doc":"The `ws` parser matches a whitespace character from the input (it is equivalent to the \\s regular expression). If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the character code of the alpha character that has been matched. Otherwise it returns `status: {:error, :unexpected_char}`.","ref":"Ergo.html#module-ws","title":"ws - Ergo","type":"module"},{"doc":"The `literal` parser is given a binary string and attempts to match it against the input. If successful it returns a context with `status: :ok` with the `char` and `ast` parameters set to the string being matched. Otherwise it returns `{:error, :unexpected_char}` for the first character that doesn't match the input.\n\nExample: literal(\"Ergo\") matches the characters 'E', 'r', 'g', and 'o' from the input successively.","ref":"Ergo.html#module-literal-s","title":"literal(s) - Ergo","type":"module"},{"doc":"These are parsers that build on the terminal parsers to parse diffrent types of numeric values.\nThe utility parsers are technically terminal parsers, since they are not parameterised by a parser, but they are implemented in terms of some of the combinator parsers themselves and hence belong in a separate category.","ref":"Ergo.html#module-numeric-parsers","title":"Numeric parsers - Ergo","type":"module"},{"doc":"The `digits` parser matches a series of digits from the input. If successful it returns with `status: :ok` and the `ast` set to a list of the digit values.","ref":"Ergo.html#module-digits","title":"digits - Ergo","type":"module"},{"doc":"The `uint` parser matches a series of digits from the input. If successful it returns with `status: :ok` and `ast` parameter set to the integer value of the number matched.","ref":"Ergo.html#module-uint","title":"uint - Ergo","type":"module"},{"doc":"The `decimal` parser matches a series of digits, separated by a single '.' from the input. If successful it returns wtih `status: :ok` and `ast` set to the floating point value of the number matched.","ref":"Ergo.html#module-decimal","title":"decimal - Ergo","type":"module"},{"doc":"The `number` parser builds upon the `uint` and `decimal` parsers to parse any kind of numeric value returning `status: :ok` and `ast` set to the integer or floating value of the number parsed.","ref":"Ergo.html#module-number","title":"number - Ergo","type":"module"},{"doc":"The combinator parsers are structural parsers that are parameterised with other parsers to match more complex structures.","ref":"Ergo.html#module-combinator-parsers","title":"Combinator Parsers - Ergo","type":"module"},{"doc":"The `sequence/2` parser is used to match a given set of parsers in sequence.\n\nExample:\n\n    p = sequence([literal(\"Hello\"), char(?\\s), literal(\"World\")])\n    p.(Context.new(\"Hello World\")) => %{status: :ok, ast: [\"Hello\", ' ', \"World\"]}\n\n    The parser p will match first the literal \"Hello\" then a single space and then the literal \"World\". If any of these parsers fail, the sequence parser will fail. This example would more simply be written literal(\"Hello World\") but more complex examples will use other combinator parsers in the sequence.\n\nOptional arguments:\n\n    ast: fn ast -> ast\n    ctx: fn ctx -> ctx\n\n    Provide a function that transforms the sequence ast into another form. For example to transform the elements of the list ast into a record type.\n\n    debug: true|false\n\n    When the sequence parser runs it logs debugging information.\n\n    label: < >\n\n    Provide a label that can be logged by setting debug: true","ref":"Ergo.html#module-sequence","title":"sequence - Ergo","type":"module"},{"doc":"The `choice/2` parser is used to match one from a sequence of parsers. It attempts to match the input against each parser in turn. The first parser that matches will cause `choice` to succeed with the `ast` set to the `ast` of the matching parser. If all of the parsers fail to match the input then `choice` will fail.\n\nSignature:\n    choice([parser1 | [parser2 | ...]], args \\\\ [])\n\nExample:\n\n```elixir\n    p = choice([literal(\"Hello\"), literal(\"World\")])\n    p.(Context.new(\"Hello World\")) -> %{status: :ok, ast: \"Hello\", input: \" World\"}\n    p.(Context.new(\"World Hello\")) -> %{status: :ok, ast: \"World\", input: \" Hello\"}\n\n    boolean = choice([literal(\"true\"), literal(\"false\")], ast: fn ast -> ast == \"true\" end)\n    boolean.(Context.new(\"true\")) -> %{status: ok, ast: true, input: \"\"}\n    boolean.(Context.new(\"false)) -> %{status: ok, ast: false, input: \"\"}\n    boolean.(Context.new(\"Hello)) -> %{status: {:error, ...}, input: \"Hello\"}\n```","ref":"Ergo.html#module-choice","title":"choice - Ergo","type":"module"},{"doc":"The `many/2` parser is used to match another parser repeatedly on the input. In its default form that can be zero times (i.e. no matches) or infinite matches. However using the `:min` and `:max` optional arguments allow limits to be specified.\n\nSignature:\n    many(parser, args \\\\ [])\n\nExample:\n\n```elixir\n    p = many(char(?\\s), min: 1)\n    p.(Context.new(\"    \")) -> %{status: :ok, input: \"\"}\n\n    p = many(char(?\\s), min: 2)\n    p.(Context.new(\" Hello\")) -> %{status: {:error, ...}, input: \" Hello\"}\n\n    p = many(char(?a..?z), max: 2)\n    p.(Context.new(\"Hello World)) -> %{status: :ok, ast: ['H', 'e'], input: \"llo World\"}\n\n    p = many(char(?a..?z), ast: fn ast -> Enum.count(ast) end)\n    p.(Context.new(\"Hello\")) -> %{status: :ok, ast: 5, input: \"\"}\n```","ref":"Ergo.html#module-many","title":"many - Ergo","type":"module"},{"doc":"The `optional/1` parser is used to match another parser on the input and succeeds if it doesn't match or matches once.","ref":"Ergo.html#module-optional","title":"optional - Ergo","type":"module"},{"doc":"The `ignore/1` parser is designed for use with parsers such as `sequence` and `many`. It takes a parser and attempts to match it on the input however, if it succeeds, it returns a nil `ast` value that parsers can use to ignore it.","ref":"Ergo.html#module-ignore","title":"ignore - Ergo","type":"module"},{"doc":"The `transform/2` parser is used to change the `ast` returned by another parser if the other parser is successful. It is useful for modifying the output of parsers that do not directly support a `map` argument.\n\nSignature:\n    squared = transform(uint(), fn ast -> ast * ast end)\n\n```elixir\nsquared.(Context.new(\"10\")) -> %{status: :ok, ast: 100}\n```","ref":"Ergo.html#module-transform","title":"transform - Ergo","type":"module"},{"doc":"The `lookahead/1` parser attempts to match the parser it is given on the input. If it succeeds it returns with `status: :ok` but `ast: nil` and with the input unchanged. Otherwise it returns with `status: {:error, :lookahead_fail}`.\n\n```elixir\np = lookahead(literal(\"Hello\"))\np.(Context.new(\"Hello World\")) -> %{status: :ok, input: \"Hello World\"}\n```","ref":"Ergo.html#module-lookahead","title":"lookahead - Ergo","type":"module"},{"doc":"The `not_lookahead/1` parser is the inverse of the `looakahead` parser in that it attempts to match its parser on the input and where it can do so it returns with `status: {:error, :lookahead_fail}`.\n\n```elixir\np = not_lookhead(literal(\"Hello\"))\np.(Context.new(\"Hello World\")) -> %{status: {:error, :lookahead_fail}}\n```","ref":"Ergo.html#module-not_lookahed","title":"not_lookahed - Ergo","type":"module"},{"doc":"The `parser/2` function is a simple entry point to parsing inputs that constructs the Context record required.\n\nOptions\n  debug: [true | false]\n\n# Examples\n\n    iex> alias Ergo.Terminals\n    iex> parser = Terminals.literal(\"Hello\")\n    iex> assert %Ergo.Context{status: :ok, ast: \"Hello\", input: \" World\", index: 5, line: 1, col: 6} = Ergo.parse(parser, \"Hello World\")","ref":"Ergo.html#parse/3","title":"Ergo.parse/3","type":"function"},{"doc":"`Ergo.Combinators` is the key set of parsers used for combining together other parsers.\n\n# Parsers\n\n* choice\n* sequence\n* many\n* optional\n* ignore\n* transform\n* lookeahead\n* not_lookahead","ref":"Ergo.Combinators.html","title":"Ergo.Combinators","type":"module"},{"doc":"The atom/1 parser takes a parser that returns an AST which is a string and\nconverts the AST into an atom.\n\n# Examples\n    iex> alias Ergo\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = many(wc()) |> string() |> atom()\n    iex> assert %Context{status: :ok, ast: :fourty_two} = Ergo.parse(parser, \"fourty_two\")","ref":"Ergo.Combinators.html#atom/1","title":"Ergo.Combinators.atom/1","type":"function"},{"doc":"The `choice/2` parser takes a list of parsers and a list (defaults to []) of\noptions. It tries to match each in turn, returning :ok if it does. Otherwise,\nif none match, it returns :error.\n\nValid options are:\n* `label: \"text\"` a label output in debugging\n  `ast: fn` a function passed the AST value for processing if the parser succeeds\n  `ctx: fn` a function passed the `Context` for processing if the parser succeeds\n  `err: fn` a function passed the `Context` for processing if the parser fails","ref":"Ergo.Combinators.html#choice/2","title":"Ergo.Combinators.choice/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = choice([literal(\"Foo\"), literal(\"Bar\"), literal(\"Hello\"), literal(\"World\")], label: \"Foo|Bar|Hello|World\")\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: \"Hello\", input: \" World\", index: 5, col: 6} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = choice([literal(\"Hello\"), literal(\"World\")], ast: &String.upcase/1)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: \"HELLO\"} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> fun = fn %Context{ast: ast} = ctx -> %{ctx | ast: String.upcase(ast)} end\n    iex> parser = choice([literal(\"Hello\"), literal(\"World\")], ctx: fun)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: \"HELLO\"} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = choice([literal(\"Foo\"), literal(\"Bar\")], label: \"Foo|Bar\")\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> %Context{status: {:error, [{:no_valid_choice, {1, 1}, \"Foo|Bar did not find a valid choice\"}]}, ast: nil, input: \"Hello World\"} = context","ref":"Ergo.Combinators.html#choice/2-examples","title":"Examples - Ergo.Combinators.choice/2","type":"function"},{"doc":"The `hoist/1` parser takes a parser expected to return an AST which is a\n1-item list. The returned parser extracts the item from the list and\nreturns an AST of just that item.\n\nThis often comes up with the sequence/2 parser and ignore, where all but\none item in a sequence are ignored. Using hoist pulls that item up so that\nsubsequent parsers don't need to deal with the list.\n\n# Examples\n    iex> alias Ergo\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = sequence([ignore(many(char(?a))), char(?b)]) |> hoist()\n    iex> assert %Context{status: :ok, ast: ?b} = Ergo.parse(parser, \"aaaaaaaab\")","ref":"Ergo.Combinators.html#hoist/1","title":"Ergo.Combinators.hoist/1","type":"function"},{"doc":"The ignore/1 parser matches but ignores the AST of its child parser.","ref":"Ergo.Combinators.html#ignore/2","title":"Ergo.Combinators.ignore/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = sequence([literal(\"Hello\"), ignore(ws()), literal(\"World\")])\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: [\"Hello\", \"World\"], index: 11, col: 12} = context","ref":"Ergo.Combinators.html#ignore/2-examples","title":"Examples - Ergo.Combinators.ignore/2","type":"function"},{"doc":"The lazy/1 parser is intended for use in cases where constructing parsers\ncreates a recursive call. By using `lazy` the original parser call is\ndeferred until later, breaking the infinite recursion.","ref":"Ergo.Combinators.html#lazy/1","title":"Ergo.Combinators.lazy/1","type":"macro"},{"doc":"The `lookahead` parser accepts a parser and matches it but does not update the context when it succeeds.","ref":"Ergo.Combinators.html#lookahead/2","title":"Ergo.Combinators.lookahead/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = lookahead(literal(\"Hello\"))\n    iex> assert %Context{status: :ok, ast: nil, input: \"Hello World\", index: 0} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = lookahead(literal(\"Helga\"))\n    iex> assert %Context{status: {:error, [{:lookahead_fail, {1, 4}, _}, {:bad_literal, {1, 4}, _}, {:unexpected_char, {1, 4}, _}]}, index: 3, col: 4, input: \"lo World\"} = Ergo.parse(parser, \"Hello World\")","ref":"Ergo.Combinators.html#lookahead/2-example","title":"Example - Ergo.Combinators.lookahead/2","type":"function"},{"doc":"The `many/2` parser takes a parser and attempts to match it on the input\nmultiple (including possibly zero) times. Accepts options `min:` and `max:`\nto modify how many times the included parser must match for `many` to\nmatch.","ref":"Ergo.Combinators.html#many/2","title":"Ergo.Combinators.many/2","type":"function"},{"doc":"This test will need to be rewritten in terms of Ergo.diganose\n    # iex> Logger.disable(self())\n    # iex> alias Ergo.Context\n    # iex> import Ergo.{Combinators, Terminals}\n    # iex> parser = many(wc(), label: \"Chars\")\n    # iex> context = Ergo.parse(parser, \"Hello World\")\n    # iex> assert %Context{status: :ok, ast: [?H, ?e, ?l, ?l, ?o], input: \" World\", index: 5, col: 6, char: ?o} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = many(wc(), min: 6)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: {:error, [{:many_less_than_min, {1, 6}, \"5 < 6\"}]}, ast: nil, input: \" World\", index: 5, col: 6} = context\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = many(wc(), max: 3)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: [?H, ?e, ?l], input: \"lo World\", index: 3, col: 4} = context\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = many(wc(), ast: &Enum.count/1)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: 5, input: \" World\", index: 5, col: 6} = context","ref":"Ergo.Combinators.html#many/2-examples","title":"Examples - Ergo.Combinators.many/2","type":"function"},{"doc":"The `not_lookahead` parser accepts a parser and attempts to match it. If the match fails the not_lookahead parser returns status: :ok but does not affect the context otherwise.\n\nIf the match succeeds the `not_lookahead` parser fails with {:error, :lookahead_fail}","ref":"Ergo.Combinators.html#not_lookahead/2","title":"Ergo.Combinators.not_lookahead/2","type":"function"},{"doc":"iex> alias Ergo.Context\n  iex> import Ergo.{Combinators, Terminals}\n  iex> parser = not_lookahead(literal(\"Foo\"))\n  iex> assert %Context{status: :ok, input: \"Hello World\"} = Ergo.parse(parser, \"Hello World\")\n\n  iex> alias Ergo.{Context, Parser}\n  iex> import Ergo.{Combinators, Terminals}\n  iex> parser = not_lookahead(literal(\"Hello\"))\n  iex> assert %Context{status: {:error, [{:lookahead_fail, {1,6}, \"Satisfied: literal \"}]}, input: \" World\"} = Ergo.parse(parser, \"Hello World\")","ref":"Ergo.Combinators.html#not_lookahead/2-examples","title":"Examples - Ergo.Combinators.not_lookahead/2","type":"function"},{"doc":"The optional/2 parser takes a parser and attempts to match it and succeeds\nwhether or not it can.","ref":"Ergo.Combinators.html#optional/2","title":"Ergo.Combinators.optional/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> ctx = Ergo.parse(optional(literal(\"Hello\")), \"Hello World\")\n    iex> assert %Context{status: :ok, ast: \"Hello\", input: \" World\", index: 5, col: 6} = ctx\n\n    In this example we deliberately ensure that the Context ast is not nil\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.{Terminals, Combinators}\n    iex> ctx = Context.new(\" World\", ast: [])\n    iex> parser = optional(literal(\"Hello\"))\n    iex> new_context = Parser.invoke(ctx, parser)\n    iex> assert %Context{status: :ok, ast: nil, input: \" World\", index: 0, col: 1} = new_context","ref":"Ergo.Combinators.html#optional/2-examples","title":"Examples - Ergo.Combinators.optional/2","type":"function"},{"doc":"","ref":"Ergo.Combinators.html#parse_many/5","title":"Ergo.Combinators.parse_many/5","type":"function"},{"doc":"The replace/3 combinator replaces the AST value of it's child with a constant.","ref":"Ergo.Combinators.html#replace/3","title":"Ergo.Combinators.replace/3","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> alias Ergo\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = ignore(literal(\"foo\")) |> replace(:foo)\n    iex> assert %Context{status: {:error, _}} = Ergo.parse(parser, \"flush\")","ref":"Ergo.Combinators.html#replace/3-examples","title":"Examples - Ergo.Combinators.replace/3","type":"function"},{"doc":"The satisfy/3 parser takes a parser and a predicate function. If the parser\nis successful the AST is passed to the predicate function. If the predicate\nfunction returns true the parser returns the successful context, otherwise\nan error context is returned.\n\n# Example\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators, Numeric}\n    iex> parser = satisfy(any(), fn char -> char in (?0..?9) end, label: \"digit char\")\n    iex> assert %Context{status: :ok, ast: ?4} = Ergo.parse(parser, \"4\")\n    iex> assert %Context{status: {:error, [{:unsatisfied, {1, 1}, \"Failed to satisfy: digit char\"}]}} = Ergo.parse(parser, \"!\")\n    iex> parser = satisfy(number(), fn n -> Integer.mod(n, 2) == 0 end, label: \"even number\")\n    iex> assert %Context{status: :ok, ast: 42} = Ergo.parse(parser, \"42\")\n    iex> assert %Context{status: {:error, [{:unsatisfied, {1, 1}, \"Failed to satisfy: even number\"}]}} = Ergo.parse(parser, \"27\")","ref":"Ergo.Combinators.html#satisfy/3","title":"Ergo.Combinators.satisfy/3","type":"function"},{"doc":"The `sequence/2` parser takes a list of parsers which are applied in turn.\nIf any of the parsers fails the sequence itself fails.\n\nThe `sequence/2` parser treats the `commit/0` parser differently. When it\nsees commit it increments the contexts commit count. When the parser\nis complete, if it has increased the commit count it decrements it before\nreturning it. See also `many`","ref":"Ergo.Combinators.html#sequence/2","title":"Ergo.Combinators.sequence/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = sequence([literal(\"Hello\"), ws(), literal(\"World\")])\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    %Context{status: :ok, ast: [\"Hello\", ?\\s, \"World\"], index: 11, line: 1, col: 12} = context\n\n    This test will need to be rewritten in terms of Ergo.diagnose\n    # iex> Logger.disable(self())\n    # iex> alias Ergo.Context\n    # iex> import Ergo.{Terminals, Combinators}\n    # iex> parser = sequence([literal(\"Hello\"), ws(), literal(\"World\")], label: \"HelloWorld\")\n    # iex> context = Ergo.parse(parser, \"Hello World\")\n    # iex> assert %Context{status: :ok, ast: [\"Hello\", ?\\s, \"World\"], index: 11, line: 1, col: 12} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = sequence([literal(\"Hello\"), ws(), literal(\"World\")], ast: fn ast -> Enum.join(ast, \" \") end)\n    iex> context = Ergo.parse(parser, \"Hello World\")\n    iex> assert %Context{status: :ok, ast: \"Hello 32 World\", index: 11, line: 1, col: 12} = context\n\n    This test will need to be rewritten in terms of Ergo.diagnose\n    # iex> Logger.disable(self())\n    # iex> alias Ergo.Context\n    # iex> import Ergo.{Terminals, Combinators}\n    # iex> parser = sequence([literal(\"Hello\"), ws(), literal(\"World\")], label: \"HelloWorld\", ast: fn ast -> Enum.join(ast, \" \") end)\n    # iex> context = Ergo.parse(parser, \"Hello World\")\n    # iex> assert %Context{status: :ok, ast: \"Hello 32 World\", index: 11, line: 1, col: 12} = context\n\n    iex> alias Ergo.Context\n    iex> import Ergo.{Combinators, Terminals}\n    iex> parser = sequence([literal(\"foo\"), ws(), literal(\"bar\")])\n    iex> assert %Context{status: {:error, [{:bad_literal, _, _}, {:unexpected_char, _, _}]}} = Ergo.parse(parser, \"Hello World\")","ref":"Ergo.Combinators.html#sequence/2-examples","title":"Examples - Ergo.Combinators.sequence/2","type":"function"},{"doc":"The string/1 parser takes a parser that returns an AST which is a list of characters\nand converts the AST into a string.\n\n# Examples\n    iex> alias Ergo\n    iex> alias Ergo.Context\n    iex> import Ergo.{Terminals, Combinators}\n    iex> parser = many(alpha()) |> string()\n    iex> assert %Context{status: :ok, ast: \"FourtyTwo\"} = Ergo.parse(parser, \"FourtyTwo\")","ref":"Ergo.Combinators.html#string/1","title":"Ergo.Combinators.string/1","type":"function"},{"doc":"The `transform/2` parser runs a transforming function on the AST of its child parser.","ref":"Ergo.Combinators.html#transform/3","title":"Ergo.Combinators.transform/3","type":"function"},{"doc":"# Sum the digits\n    iex> alias Ergo.Context\n    iex> import Ergo.{Combinators, Terminals}\n    iex> digit_to_int = fn d -> List.to_string([d]) |> String.to_integer() end\n    iex> t_fn = fn ast -> ast |> Enum.map(digit_to_int) |> Enum.sum() end\n    iex> parser = sequence([digit(), digit(), digit(), digit()]) |> transform(t_fn)\n    iex> context = Ergo.parse(parser, \"1234\")\n    iex> %Context{status: :ok, ast: 10, index: 4, line: 1, col: 5} = context","ref":"Ergo.Combinators.html#transform/3-examples","title":"Examples - Ergo.Combinators.transform/3","type":"function"},{"doc":"`Ergo.Context` defines the `Context` struct that is used to maintain parser state as the various\nparsers work, and various functions for creating & manipulating contexts.\n\n# Fields\n\n* `status`\n\nWhen a parser returns it either sets `status` to `:ok` to indicate that it was successful or to a tuple\n`{:error, reasons}` where reasons is a list of tuples representing a sequence of errors. Each tuple is of\nthe form `{code, message}` where code is an atom indiciting the specific type of error and the message\ncontains additional human-focused information that may be helpful in diagnosing the problem.\n\n* `input`\n\nThe binary input being parsed.\n\n* `consumed`\n\nThe binary input that has already been consumed by parsing.\n\n* `index`\n\nRepresents the position in the input which has been read so far. Initially 0 it increments for each character processed.\n\n* `line`\n\nRepresents the current line of the input. Initially 1 it increments whenever a `\n` is read from the input.\n\n* `col`\n\nRepresents the current column of the input. Initially 1 it is incremented every time a character is read from the input and automatically resets whenever a `\n` is read.\n\n* `entry_points`\n\nA list of `Parser` entry points\n\n* `data`\n\nMap containing user-data that the parsers can use to pass information between them.\n\n* `ast`\n\nRepresents the current data structure being built from the input.\n\n* `parser`\n\nThe current parser.\n\n* `tracks`\n\nParsers will add themselves to the `Context` tracks in the form of `{ref, index}`.\nIf the same parser attempts to add itself a second time at the same index an\nerror is thrown because a cycle has been detected.\n\n* `depth`\n\nAs nested parsers are called the depth field will be updated to reflect the\nnumber of levels of nesting for the current parser.\n\n* `captures`\n\nA map of data that can be captured from ASTs using the capture parser.","ref":"Ergo.Context.html","title":"Ergo.Context","type":"module"},{"doc":"","ref":"Ergo.Context.html#add_error/3","title":"Ergo.Context.add_error/3","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> context =\n    ...>  Context.new(\"Hello World\")\n    ...>  |> Context.add_error(:unexpected_char, \"Expected: |e| Actual: |.|\")\n    iex> assert is_nil(context.ast)\n    iex> assert {:error, [{:unexpected_char, {1, 1}, \"Expected: |e| Actual: |.|\"}]} = context.status\n\n    iex> alias Ergo.Context\n    iex> context =\n    ...>  Context.new(\"Hello World\")\n    ...>  |> Context.add_error(:unexpected_char, \"Expected: |e| Actual: |.|\")\n    ...>  |> Context.add_error(:literal_failed, \"Expected 'end'\")\n    iex> assert is_nil(context.ast)\n    iex> assert {:error, [{:literal_failed, {1, 1}, \"Expected 'end'\"}, {:unexpected_char, {1, 1}, \"Expected: |e| Actual: |.|\"}]} = context.status","ref":"Ergo.Context.html#add_error/3-examples","title":"Examples - Ergo.Context.add_error/3","type":"function"},{"doc":"Because we build ASTs using lists they end up in reverse order. This method reverses the AST back\nto in-parse-order","ref":"Ergo.Context.html#ast_in_parsed_order/1","title":"Ergo.Context.ast_in_parsed_order/1","type":"function"},{"doc":"iex> context = Ergo.Context.new(\"\", ast: [4, 3, 2, 1])\n    iex> assert %Context{ast: [1, 2, 3, 4]} = Context.ast_in_parsed_order(context)","ref":"Ergo.Context.html#ast_in_parsed_order/1-examples","title":"Examples - Ergo.Context.ast_in_parsed_order/1","type":"function"},{"doc":"Where an AST has been built from individual characters and needs to be converted to a string","ref":"Ergo.Context.html#ast_to_string/1","title":"Ergo.Context.ast_to_string/1","type":"function"},{"doc":"iex> context = Ergo.Context.new(\"\", ast: [?H, ?e, ?l, ?l, ?o])\n    iex> assert %Context{ast: \"Hello\"} = Context.ast_to_string(context)","ref":"Ergo.Context.html#ast_to_string/1-examples","title":"Examples - Ergo.Context.ast_to_string/1","type":"function"},{"doc":"Called to perform an arbitrary transformation on the AST value of a Context.","ref":"Ergo.Context.html#ast_transform/2","title":"Ergo.Context.ast_transform/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> context = Context.new(\"\", ast: \"Hello World\")\n    iex> assert %Context{ast: \"Hello World\"} = Context.ast_transform(context, &Function.identity/1)\n\n    iex> alias Ergo.Context\n    iex> context = Context.new(\"\", ast: \"Hello World\")\n    iex> assert %Context{ast: 11} = Context.ast_transform(context, &String.length/1)\n\n    iex> alias Ergo.Context\n    iex> context = Context.new(\"\", ast: \"Hello World\")\n    iex> assert %Context{ast: \"Hello World\"} = Context.ast_transform(context, nil)","ref":"Ergo.Context.html#ast_transform/2-examples","title":"Examples - Ergo.Context.ast_transform/2","type":"function"},{"doc":"The `ignore` parser matches but returns a nil for the AST. Parsers like `sequence` accumulate these nil values.\nCall this function to remove them","ref":"Ergo.Context.html#ast_without_ignored/1","title":"Ergo.Context.ast_without_ignored/1","type":"function"},{"doc":"iex> context = Ergo.Context.new(\"\", ast: [\"Hello\", nil, \"World\", nil])\n    iex> assert %Context{ast: [\"Hello\", \"World\"]} = Context.ast_without_ignored(context)","ref":"Ergo.Context.html#ast_without_ignored/1-examples","title":"Examples - Ergo.Context.ast_without_ignored/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#clip/2","title":"Ergo.Context.clip/2","type":"function"},{"doc":"","ref":"Ergo.Context.html#commit/1","title":"Ergo.Context.commit/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#dec_depth/1","title":"Ergo.Context.dec_depth/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#inc_depth/1","title":"Ergo.Context.inc_depth/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#make_error_fatal/1","title":"Ergo.Context.make_error_fatal/1","type":"function"},{"doc":"`new` returns a newly initialised `Context` with `input` set to the string passed in.","ref":"Ergo.Context.html#new/2","title":"Ergo.Context.new/2","type":"function"},{"doc":"iex> alias Ergo.Context\n  iex> assert %Context{status: :ok, input: \"Hello World\", line: 1, col: 1, index: 0, tracks: %{}} = Context.new(\"Hello World\")","ref":"Ergo.Context.html#new/2-examples","title":"Examples: - Ergo.Context.new/2","type":"function"},{"doc":"Reads the next character from the `input` of the passed in `Context`.\n\nIf the `input` is empty returns `status: {:error, :unexpected_eoi}`.\n\nOtherwise returns a new `Context` setting `ast` to the character read and incrementing positional variables such as `index`, `line`, and `column` appropriately.","ref":"Ergo.Context.html#next_char/1","title":"Ergo.Context.next_char/1","type":"function"},{"doc":"iex> context = Context.next_char(Context.new(\"\"))\n  iex> assert %Context{status: {:error, [{:unexpected_eoi, {1, 1}, \"Unexpected end of input\"}] }} = context\n\n  iex> context = Context.next_char(Context.new(\"Hello World\"))\n  iex> assert %Context{status: :ok, input: \"ello World\", ast: ?H, index: 1, line: 1, col: 2} = context","ref":"Ergo.Context.html#next_char/1-examples","title":"Examples - Ergo.Context.next_char/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#parent_parser/1","title":"Ergo.Context.parent_parser/1","type":"function"},{"doc":"Returns truthy value if the parser referred to by `ref` has already been called for the index `index`.","ref":"Ergo.Context.html#parser_tracked?/2","title":"Ergo.Context.parser_tracked?/2","type":"function"},{"doc":"iex> alias Ergo.Context\n  iex> parser_ref = 123\n  iex> context = Context.new(\"Hello World\") |> Context.track_parser(parser_ref, :foo)\n  iex> assert Context.parser_tracked?(context, parser_ref)","ref":"Ergo.Context.html#parser_tracked?/2-examples","title":"Examples - Ergo.Context.parser_tracked?/2","type":"function"},{"doc":"","ref":"Ergo.Context.html#peek/1","title":"Ergo.Context.peek/1","type":"function"},{"doc":"iex> context = Context.new(\"Hello\")\n    iex> assert %Context{status: :ok, ast: ?H, input: \"ello\", index: 1, line: 1, col: 2} = Context.peek(context)\n\n    iex> context = Context.new(\"\")\n    iex> assert %Context{status: {:error, [{:unexpected_eoi, {1, 1}, \"Unexpected end of input\"}]}, index: 0, line: 1, col: 1} = Context.peek(context)","ref":"Ergo.Context.html#peek/1-examples","title":"Examples - Ergo.Context.peek/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#pop_parser/1","title":"Ergo.Context.pop_parser/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#push_parser/2","title":"Ergo.Context.push_parser/2","type":"function"},{"doc":"Clears the value of the status and ast fields to ensure that the wrong status information cannot be returned from a child parser.","ref":"Ergo.Context.html#reset_status/1","title":"Ergo.Context.reset_status/1","type":"function"},{"doc":"iex> context = Context.new(\"Hello World\")\n    iex> context = %{context | status: {:error, [{:inexplicable_error, \"What the…\"}]}, ast: true}\n    iex> context = Context.reset_status(context)\n    iex> assert %Context{status: :ok, ast: nil} = context","ref":"Ergo.Context.html#reset_status/1-examples","title":"Examples - Ergo.Context.reset_status/1","type":"function"},{"doc":"","ref":"Ergo.Context.html#set_ast/2","title":"Ergo.Context.set_ast/2","type":"function"},{"doc":"Updates the `Context` to track that the parser referred to by `ref` has been called for the index `index`.\n\nExamples:\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> context = Context.new(\"Hello World\")\n    iex> parser = literal(\"Hello\")\n    iex> context2 = Context.track_parser(context, parser.ref, :foo)\n    iex> assert Map.has_key?(context2.tracks, {0, parser.ref})","ref":"Ergo.Context.html#track_parser/3","title":"Ergo.Context.track_parser/3","type":"function"},{"doc":"","ref":"Ergo.Context.html#uncommit/1","title":"Ergo.Context.uncommit/1","type":"function"},{"doc":"The Meta parsers are not really parsers at all but operate within the parsing framework.\n\nAll Meta parsers are combinators that accept a parser and either report on or modify\nits operation.","ref":"Ergo.Meta.html","title":"Ergo.Meta","type":"module"},{"doc":"The `around` parser is a pass-thru parser that can be used to run code before or\nafter another parser. For example to inspect the context or output additional\ndebugging information.\n\nSpecify on of\n  around(parser, before: fn ctx -> … end)\n  around(parser, after: fn ctx, parsed_ctx -> … end)\n  around(parser, before: fn ctx -> … end, after: fn ctx, parsed_ctx -> … end)\n\nNote that where both `before:` and `after:` are specified, the after function will\nreceive both the context before and after parsing as separate parameters.\n\nIf neither of `before:` nor `around:` are specified an error will be raised.\n\nExamples\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Meta\n    iex> null_parser = Parser.combinator(:null, \"null_parser\", fn %Context{} = ctx -> ctx end)\n    iex> assert_raise(RuntimeError, fn -> Ergo.parse(around(null_parser), \"\") end)\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Meta\n    iex> null_parser = Parser.combinator(:null, \"null_parser\", fn %Context{} = ctx -> ctx end)\n    iex> parser = around(null_parser, before: fn _ctx -> send(self(), :before) end)\n    iex> Ergo.parse(parser, \"\")\n    iex> assert_receive :before\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Meta\n    iex> null_parser = Parser.combinator(:null, \"null_parser\", fn %Context{} = ctx -> ctx end)\n    iex> parser = around(null_parser, after: fn _ctx, _new_ctx -> send(self(), :after) end)\n    iex> Ergo.parse(parser, \"\")\n    iex> assert_receive :after\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Meta\n    iex> null_parser = Parser.combinator(:null, \"null_parser\", fn %Context{} = ctx -> ctx end)\n    iex> parser = around(null_parser, before: fn _ctx -> send(self(), :before) end, after: fn _ctx, _new_ctx -> send(self(), :after) end)\n    iex> Ergo.parse(parser, \"\")\n    iex> assert_receive :before\n    iex> assert_receive :after","ref":"Ergo.Meta.html#around/2","title":"Ergo.Meta.around/2","type":"function"},{"doc":"`capture/2` is used to hoist AST values (possibly transformed by the t_fn)\ninto the context. For example this can be used in a sequence where you want\nto communicate information forward in the sequence.\n\nExample: You want to parse  ... . To correctly parse   you need to\nknow that   was used to open the pair.","ref":"Ergo.Meta.html#capture/3","title":"Ergo.Meta.capture/3","type":"function"},{"doc":"`commit/0` is not technically a parser, rather a sentinel that a sequence\ncan use to determine whether an error is fatal or not","ref":"Ergo.Meta.html#commit/0","title":"Ergo.Meta.commit/0","type":"function"},{"doc":"The `failed` parser is a combinator that invokes a parser and if the parser fails\nruns the given function on the resulting context. This can be used to output additional\ndebugging information where failure was unexpected.\n\nExamples\n\n    iex> alias Ergo.Parser\n    iex> import Ergo.Meta\n    iex> failing_parser = Parser.combinator(:err, \"err_parser\", fn ctx -> %{ctx | status: {:error, :unfathomable_error}} end)\n    iex> parser = failed(failing_parser, fn _pre_ctx, _post_ctx, _parser -> send(self(), :failed) end)\n    iex> Ergo.parse(parser, \"\")\n    iex> assert_received :failed","ref":"Ergo.Meta.html#failed/3","title":"Ergo.Meta.failed/3","type":"function"},{"doc":"The Parsers module exists to house utility parsers that while they are terminals in the sense that they are not parameterised, they internally make use of parsers from the Combinators module.\n\n# Parsers\n\n* `uint`\n* `decimal`\n* `digits`","ref":"Ergo.Numeric.html","title":"Ergo.Numeric","type":"module"},{"doc":"","ref":"Ergo.Numeric.html#decimal/1","title":"Ergo.Numeric.decimal/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Numeric\n    iex> context = Ergo.parse(decimal(), \"234.56\")\n    iex> assert %Context{status: :ok, ast: 234.56} = context","ref":"Ergo.Numeric.html#decimal/1-examples","title":"Examples - Ergo.Numeric.decimal/1","type":"function"},{"doc":"The `digits` parser matches a series of at least one digit and returns an enumeration\nof the digits.","ref":"Ergo.Numeric.html#digits/1","title":"Ergo.Numeric.digits/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Numeric\n    iex> context = Ergo.parse(digits(), \"2345\")\n    iex> assert %Context{status: :ok, ast: [2, 3, 4, 5]} = context","ref":"Ergo.Numeric.html#digits/1-examples","title":"Examples - Ergo.Numeric.digits/1","type":"function"},{"doc":"The `number` parser matches both integer and decimal string and converts them into their\nappropriate Elixir integer or float values.","ref":"Ergo.Numeric.html#number/1","title":"Ergo.Numeric.number/1","type":"function"},{"doc":"iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 42} = Ergo.parse(number(), \"42\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 42} = Ergo.parse(number(), \"+42\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: -42} = Ergo.parse(number(), \"-42\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 42.0} = Ergo.parse(number(), \"42.0\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: -42.0} = Ergo.parse(number(), \"-42.0\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 0} = Ergo.parse(number(), \"0\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 0} = Ergo.parse(number(), \"0000\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: {:error, _}} = Ergo.parse(number(), \"Fourty Two\")\n\n    iex> import Ergo.Numeric\n    iex> assert %{status: :ok, ast: 42} = Ergo.parse(number(), \"42Fourty Two\")","ref":"Ergo.Numeric.html#number/1-examples","title":"Examples - Ergo.Numeric.number/1","type":"function"},{"doc":"The `unit` parser matches a series of at least one digit and returns the\ninteger value of the digits.","ref":"Ergo.Numeric.html#uint/1","title":"Ergo.Numeric.uint/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Numeric\n    iex> context = Ergo.parse(uint(), \"2345\")\n    iex> assert %Context{status: :ok, ast: 2345, index: 4, col: 5} = context","ref":"Ergo.Numeric.html#uint/1-examples","title":"Examples - Ergo.Numeric.uint/1","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html","title":"Ergo.Outline.Builder","type":"module"},{"doc":"","ref":"Ergo.Outline.Builder.html#append/2","title":"Ergo.Outline.Builder.append/2","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#build_from_events/1","title":"Ergo.Outline.Builder.build_from_events/1","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#insert/1","title":"Ergo.Outline.Builder.insert/1","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#insert/2","title":"Ergo.Outline.Builder.insert/2","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#is_branch/1","title":"Ergo.Outline.Builder.is_branch/1","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#kids/1","title":"Ergo.Outline.Builder.kids/1","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#make_node/2","title":"Ergo.Outline.Builder.make_node/2","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#root_node/0","title":"Ergo.Outline.Builder.root_node/0","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#tree/0","title":"Ergo.Outline.Builder.tree/0","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#walk/2","title":"Ergo.Outline.Builder.walk/2","type":"function"},{"doc":"","ref":"Ergo.Outline.Builder.html#walk_node/2","title":"Ergo.Outline.Builder.walk_node/2","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html","title":"Ergo.Outline.OPML","type":"module"},{"doc":"","ref":"Ergo.Outline.OPML.html#generate_node/2","title":"Ergo.Outline.OPML.generate_node/2","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#generate_opml/2","title":"Ergo.Outline.OPML.generate_opml/2","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#generate_opml_node/11","title":"Ergo.Outline.OPML.generate_opml_node/11","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_info/1","title":"Ergo.Outline.OPML.opml_event_info/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_input/1","title":"Ergo.Outline.OPML.opml_event_input/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_match/1","title":"Ergo.Outline.OPML.opml_event_match/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_name/1","title":"Ergo.Outline.OPML.opml_event_name/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_pos/2","title":"Ergo.Outline.OPML.opml_event_pos/2","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_status/1","title":"Ergo.Outline.OPML.opml_event_status/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_tagend/1","title":"Ergo.Outline.OPML.opml_event_tagend/1","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#opml_event_text/2","title":"Ergo.Outline.OPML.opml_event_text/2","type":"function"},{"doc":"","ref":"Ergo.Outline.OPML.html#to_xml_attr_value/1","title":"Ergo.Outline.OPML.to_xml_attr_value/1","type":"function"},{"doc":"`Ergo.Parser` contains the Parser record type. Ergo parsers are anonymous functions but we embed\nthem in a `Parser` record that can hold arbitrary metadata. The primary use for the metadata is\nthe storage of debugging information.","ref":"Ergo.Parser.html","title":"Ergo.Parser","type":"module"},{"doc":"","ref":"Ergo.Parser.html#child_info_for_telemetry/1","title":"Ergo.Parser.child_info_for_telemetry/1","type":"function"},{"doc":"Create a new combinator parser with the given label, parsing function, and\noptional metadata.","ref":"Ergo.Parser.html#combinator/4","title":"Ergo.Parser.combinator/4","type":"function"},{"doc":"`invoke/2` invokes the parsing function of the given parser on the specified\n`%Context{}` structure. It maintains housekeeping for the parser generally.","ref":"Ergo.Parser.html#invoke/2","title":"Ergo.Parser.invoke/2","type":"function"},{"doc":"","ref":"Ergo.Parser.html#next_ref/0","title":"Ergo.Parser.next_ref/0","type":"function"},{"doc":"","ref":"Ergo.Parser.html#pop_entry_point/1","title":"Ergo.Parser.pop_entry_point/1","type":"function"},{"doc":"","ref":"Ergo.Parser.html#push_entry_point/1","title":"Ergo.Parser.push_entry_point/1","type":"function"},{"doc":"Create a new terminal parser with the given label, parsing function, and\noptional metadata.","ref":"Ergo.Parser.html#terminal/4","title":"Ergo.Parser.terminal/4","type":"function"},{"doc":"`track_parser` first checks if the parser has already been tracked for the current input index and, if it has,\nraises a `CycleError` to indicate the parser is in a loop. Otherwise it adds the parser at the current index.","ref":"Ergo.Parser.html#track_parser/1","title":"Ergo.Parser.track_parser/1","type":"function"},{"doc":"iex> alias Ergo.{Context, Parser}\n  iex> import Ergo.{Terminals, Combinators}\n  iex> parser = many(char(?H))\n  iex> context =\n  ...>  Context.new(\"Hello World\")\n  ...>  |> Map.put(:parser, parser)\n  ...>  |> Parser.track_parser()\n  iex> assert Context.parser_tracked?(context, parser.ref)","ref":"Ergo.Parser.html#track_parser/1-examples","title":"Examples - Ergo.Parser.track_parser/1","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html","title":"Ergo.Telemetry","type":"module"},{"doc":"","ref":"Ergo.Telemetry.html#enter/1","title":"Ergo.Telemetry.enter/1","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html#event/3","title":"Ergo.Telemetry.event/3","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html#get_events/1","title":"Ergo.Telemetry.get_events/1","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html#leave/1","title":"Ergo.Telemetry.leave/1","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html#result/1","title":"Ergo.Telemetry.result/1","type":"function"},{"doc":"","ref":"Ergo.Telemetry.html#start/0","title":"Ergo.Telemetry.start/0","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html","title":"Ergo.TelemetryServer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ergo.TelemetryServer.html#child_spec/1","title":"Ergo.TelemetryServer.child_spec/1","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html#get_events/1","title":"Ergo.TelemetryServer.get_events/1","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html#handle_event/4","title":"Ergo.TelemetryServer.handle_event/4","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html#init/1","title":"Ergo.TelemetryServer.init/1","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html#reset/0","title":"Ergo.TelemetryServer.reset/0","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.html#start_link/1","title":"Ergo.TelemetryServer.start_link/1","type":"function"},{"doc":"","ref":"Ergo.TelemetryServer.State.html","title":"Ergo.TelemetryServer.State","type":"module"},{"doc":"`Ergo.Terminals` contains the terminal parsers, which are those parsers not\nparameterized with other parsers and therefore work more at the level of text\nthan structure.","ref":"Ergo.Terminals.html","title":"Ergo.Terminals","type":"module"},{"doc":"The `alpha/0` parser accepts a single character in the range a..z or A..Z.","ref":"Ergo.Terminals.html#alpha/1","title":"Ergo.Terminals.alpha/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = alpha()\n    iex> assert %Context{status: :ok, input: \"ello World\", ast: ?H, index: 1, line: 1, col: 2} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = alpha()\n    iex> assert %Context{status: :ok, input: \"llo World\", ast: ?e, index: 1, line: 1, col: 2} = Ergo.parse(parser, \"ello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = alpha()\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: [|a|..|z|, |A|..|Z|] Actual: | |\"}]}, input: \" World\"} = Ergo.parse(parser, \" World\")","ref":"Ergo.Terminals.html#alpha/1-examples","title":"Examples - Ergo.Terminals.alpha/1","type":"function"},{"doc":"The `any/0` parser matches any character.\n\n# Examples\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = any()\n    iex> assert %Context{status: :ok, ast: ?H} = Ergo.parse(parser, \"H\")\n    iex> assert %Context{status: :ok, ast: ?e} = Ergo.parse(parser, \"e\")\n    iex> assert %Context{status: :ok, ast: ?!} = Ergo.parse(parser, \"!\")\n    iex> assert %Context{status: :ok, ast: ?0} = Ergo.parse(parser, \"0\")","ref":"Ergo.Terminals.html#any/0","title":"Ergo.Terminals.any/0","type":"function"},{"doc":"","ref":"Ergo.Terminals.html#captured_literal/1","title":"Ergo.Terminals.captured_literal/1","type":"function"},{"doc":"The `char/1` parser is a terminal parser that matches a specific character.","ref":"Ergo.Terminals.html#char/2","title":"Ergo.Terminals.char/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?H)\n    iex> assert %Context{status: :ok, ast: ?H, input: \"ello World\", index: 1, line: 1, col: 2} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?h)\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: |h| Actual: |H|\"}]}, input: \"Hello World\"} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?H)\n    iex> assert %Context{status: {:error, [{:unexpected_eoi, {1, 1}, \"Unexpected end of input\"}]}} = Ergo.parse(parser, \"\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?A..?Z)\n    iex> assert %Context{status: :ok, ast: ?H, input: \"ello World\", index: 1, line: 1, col: 2} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?a..?z)\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: |a|..|z| Actual: |H|\"}]},input: \"Hello World\"} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(?A..?Z)\n    iex> assert %Context{status: {:error, [{:unexpected_eoi, {1, 1}, \"Unexpected end of input\"}]}} = Ergo.parse(parser, \"\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char([?a..?z, ?A..?Z])\n    iex> assert %Context{status: :ok, ast: ?H, input: \"ello World\", index: 1, line: 1, col: 2} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char([?a..?z, ?A..?Z])\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: [|a|..|z|, |A|..|Z|] Actual: |0|\"}]}, input: \"0000\"} = Ergo.parse(parser, \"0000\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(-?0)\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Should not have matched |0|\"}]}, input: \"0000\"} = Ergo.parse(parser, \"0000\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = char(-?a)\n    iex> assert %Context{status: :ok, input: \"000\", ast: ?0, index: 1, col: 2} = Ergo.parse(parser, \"0000\")","ref":"Ergo.Terminals.html#char/2-examples","title":"Examples - Ergo.Terminals.char/2","type":"function"},{"doc":"The delimited_text/2 parser matches a sequence of text delimited `open_char` and\n`close_char`. Because it is expected that `open_char` may appear multiple times\nwithin the sequence it balances the tokens to ensure the right number of closing\ntokens is matched.\n\n# Examples\n      iex> alias Ergo\n      iex> alias Ergo.Context\n      iex> import Ergo.Terminals\n      iex> parser = delimited_text(?{, ?})\n      iex> assert %Context{status: :ok, ast: \"{return {foo: \\\"bar\\\", bar: {baz: \\\"quux\\\"}};}\", input: \"\"} = Ergo.parse(parser, \"{return {foo: \\\"bar\\\", bar: {baz: \\\"quux\\\"}};}\")\n      iex> assert %Context{status: :ok, ast: \"{function b(y) {return x + y;}; return b;}\", input: \"foo\"} = Ergo.parse(parser, \"{function b(y) {return x + y;}; return b;}foo\")","ref":"Ergo.Terminals.html#delimited_text/3","title":"Ergo.Terminals.delimited_text/3","type":"function"},{"doc":"The `digit/0` parser accepts a character in the range of 0..9","ref":"Ergo.Terminals.html#digit/1","title":"Ergo.Terminals.digit/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = digit()\n    iex> assert %Context{status: :ok, ast: ?0, input: \"000\", index: 1, line: 1, col: 2} = Ergo.parse(parser, \"0000\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> import Ergo.Terminals\n    iex> parser = digit()\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: |0|..|9| Actual: |A|\"}]}, input: \"AAAA\", index: 0, line: 1, col: 1} = Ergo.parse(parser, \"AAAA\")\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Terminals\n    iex> ctx = Context.new(\"\")\n    iex> parser = digit()\n    iex> assert %Context{status: {:error, [{:unexpected_eoi, {1, 1}, \"Unexpected end of input\"}]}, input: \"\", index: 0, line: 1, col: 1} = Parser.invoke(ctx, parser)","ref":"Ergo.Terminals.html#digit/1-examples","title":"Examples - Ergo.Terminals.digit/1","type":"function"},{"doc":"The eoi parser is a terminal parser that checks whether the input\nhas been fully consumed. If there is input remaining to be parsed\nthe return context status is set to :error.","ref":"Ergo.Terminals.html#eoi/0","title":"Ergo.Terminals.eoi/0","type":"function"},{"doc":"iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Terminals\n    iex> ctx = Context.new(\"\")\n    iex> assert %Context{status: :ok, ast: nil} = Parser.invoke(ctx, eoi())\n\n    iex> alias Ergo.{Context, Parser}\n    iex> import Ergo.Terminals\n    iex> ctx = Context.new(\"Hello World\")\n    iex> assert %Context{status: {:error, [{:not_eoi, {1, 1}, \"Input not empty: Hello World\"}]}, input: \"Hello World\"} = Parser.invoke(ctx, eoi())","ref":"Ergo.Terminals.html#eoi/0-examples","title":"Examples - Ergo.Terminals.eoi/0","type":"function"},{"doc":"The `literal/1` parser matches the specified string character by character.","ref":"Ergo.Terminals.html#literal/2","title":"Ergo.Terminals.literal/2","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = literal(\"Hello\")\n    iex> assert %Context{status: :ok, input: \" World\", ast: \"Hello\", index: 5, line: 1, col: 6} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = literal(\"Hello\")\n    iex> assert %Context{status: {:error, [{:bad_literal, {1, 5}, \"literal \"}, {:unexpected_char, {1, 5}, \"Expected: |o| Actual: |x|\"}]}, input: \"x World\", index: 4, line: 1, col: 5} = Ergo.parse(parser, \"Hellx World\")","ref":"Ergo.Terminals.html#literal/2-examples","title":"Examples - Ergo.Terminals.literal/2","type":"function"},{"doc":"The `non_ws/1` parser accepts any character that is not a white space.","ref":"Ergo.Terminals.html#non_ws/1","title":"Ergo.Terminals.non_ws/1","type":"function"},{"doc":"The not_char matcher accepts a char or a list of chars and will match any\nchar that is not in the list.\n\n# Examples\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = not_char(?0)\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Should not have matched |0|\"}]}, input: \"0000\"} = Ergo.parse(parser, \"0000\")\n    iex> assert %Context{status: :ok, ast: ?1} = Ergo.parse(parser, \"1111\")\n    iex> parser = not_char([?{, ?}])\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Should not have matched |{|\"}]}, input: \"{}\"} = Ergo.parse(parser, \"{}\")\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Should not have matched |}|\"}]}, input: \"}\"} = Ergo.parse(parser, \"}\")","ref":"Ergo.Terminals.html#not_char/2","title":"Ergo.Terminals.not_char/2","type":"function"},{"doc":"The `wc/0` parser parses a word character and is analagous to the \\w regular expression.","ref":"Ergo.Terminals.html#wc/0","title":"Ergo.Terminals.wc/0","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = wc()\n    iex> assert %Context{status: :ok, ast: ?H, input: \"ello World\", index: 1, col: 2} = Ergo.parse(parser, \"Hello World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = wc()\n    iex> assert %Context{status: :ok, ast: ?0, input: \" World\", index: 1, col: 2} = Ergo.parse(parser, \"0 World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = wc()\n    iex> assert %Context{status: :ok, ast: ?_, input: \"Hello\", index: 1, col: 2} = Ergo.parse(parser, \"_Hello\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = wc()\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: [|0|..|9|, |a|..|z|, |A|..|Z|, |_|] Actual: | |\"}]}, input: \" Hello\"} = Ergo.parse(parser, \" Hello\")","ref":"Ergo.Terminals.html#wc/0-examples","title":"Examples - Ergo.Terminals.wc/0","type":"function"},{"doc":"The `ws/0` parser accepts a white space character and is equivalent to the \\s regular expression.","ref":"Ergo.Terminals.html#ws/1","title":"Ergo.Terminals.ws/1","type":"function"},{"doc":"iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = ws()\n    iex> assert %Context{status: :ok, ast: ?\\s, input: \"World\", index: 1, line: 1, col: 2}= Ergo.parse(parser, \" World\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = ws()\n    iex> assert %Context{status: :ok, ast: ?\\t, input: \"World\", index: 1, line: 1, col: 2} = Ergo.parse(parser, \"\\tWorld\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = ws()\n    iex> assert %Context{status: :ok, ast: ?\\n, input: \"World\", index: 1, line: 2, col: 1} = Ergo.parse(parser, \"\\nWorld\")\n\n    iex> alias Ergo.Context\n    iex> import Ergo.Terminals\n    iex> parser = ws()\n    iex> assert %Context{status: {:error, [{:unexpected_char, {1, 1}, \"Expected: [| |, |\\t|, |\\r|, |\\n|, |\\v|] Actual: |H|\"}]}, input: \"Hello World\"} = Ergo.parse(parser, \"Hello World\")","ref":"Ergo.Terminals.html#ws/1-examples","title":"Examples - Ergo.Terminals.ws/1","type":"function"},{"doc":"","ref":"Ergo.Utils.html","title":"Ergo.Utils","type":"module"},{"doc":"If the string `str` is longer than `max` (default: 40) it is truncated to 37 chars and \"...\" is added to the end, otherwise it is returned unaltered.","ref":"Ergo.Utils.html#ellipsize/2","title":"Ergo.Utils.ellipsize/2","type":"function"},{"doc":"iex> Ergo.Utils.ellipsize(\"frob\")\n    \"frob\"\n\n    iex> Ergo.Utils.ellipsize(\"123456789012345678901234567890123456789012345678901\")\n    \"1234567890123456789012345678901234567...\"\n\n    iex> Ergo.Utils.ellipsize(\"12345678901234567890\", 10)\n    \"1234567...\"\n\n    iex> Ergo.Utils.ellipsize(\"12345\", 4)\n    \"1...\"","ref":"Ergo.Utils.html#ellipsize/2-examples","title":"Examples - Ergo.Utils.ellipsize/2","type":"function"},{"doc":"","ref":"Ergo.Utils.html#format_tracks/1","title":"Ergo.Utils.format_tracks/1","type":"function"},{"doc":"","ref":"Ergo.Utils.html#printable_string/1","title":"Ergo.Utils.printable_string/1","type":"function"},{"doc":"","ref":"Ergo.Utils.html#typeof/1","title":"Ergo.Utils.typeof/1","type":"function"},{"doc":"","ref":"Ergo.Parser.CycleError.html","title":"Ergo.Parser.CycleError","type":"exception"},{"doc":"# Overview\n\nErgo is a parser combinator library written in Elixir. The name 'Ergo' comes from 'ergo' which means 'therefore' which means 'for that reason' which seemed appropriate for a parser.\n\nIf you are already familiar with parser combinators, great! If not we have a [gentle introduction](parser_combinator_intro.html) or see Saša Jurić's talk where he [builds up parser combinators from the ground up](https://www.youtube.com/watch?v=xNzoerDljjo).\n\nIf you want to know why you should (or shouldn't) use Ergo see [comparison with other parser combinator libraries](comparisons.md).","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"Add Ergo to your mix.exs file:\n\n    {:ergo, \"~> 0.9\"}\n\nThen run\n\n    mix deps.get","ref":"overview.html#how-to-install-ergo","title":"How to install Ergo. - Overview","type":"extras"},{"doc":"# Intro to Parser Combinators\n\nIf you prefer to watch to a talk see Saša Jurić [build up parser combinators from the ground up](https://www.youtube.com/watch?v=xNzoerDljjo)\n\nThe phrase __parser combinator__ may sound confusing but is in fact deceptively simple if you already understand the notion of functions and function composition.\n\nFor example transforming `[1, 2, 3, 4, 5]` into `[2, 4, 6, 8, 10]` can be achieved through combining the functions `Enum.map` and `double` as follows:\n\n    double = fn x -> 2 * x end\n    Enum.map([1, 2, 3, 4, 5], double)\n\nIf you think of a parser as being a certain kind of function that works on a textual input (in the way that `double` above is a kind of function that works on a number input) then you may see how we can combine parsers to do work.\n\nFor example to parse an input such as \"12345\" we might combine two parsers `many` and `digit`. Let's imagine digit is a parser that accepts an input character in the range of '0'..'9' and returns a number, like this:\n\n    digit(\"12345\") = 1\n    digit(\"2345\") = 2\n\nand so on. Now lets imagine that `many` is a parser that accepts an input and another parser and keeps attempting to match that parser against the input until it doesn't match any further. So, for example:\n\n    many(\"12345\", digit) = [1, 2, 3, 4, 5]\n\nIt's the \"accepts another parser\" concept that is key and is what makes `many` a combinator parser. In the same way as `map` calls the function `double` to do work, `many` calls the parser `digit` to do work.\n\nThe insight is that we can replace `digit` with any other parser including other combinator parsers. In this way we can combine, and recombine, simpler parsers to form more complex parsers until we have something that can parse our input.\n\nIn Ergo we distinguish between **terminal** parsers that only work on the input and **combinator** parsers that are parameterised with one or more other parsers (which may themselves be either a terminal or another combinator parser).","ref":"parser_combinator_intro.html","title":"Intro to Parser Combinators","type":"extras"},{"doc":"# Comparisons with other libraries\n\nThere are already at least two mature parser combinator libraries for Elixir: [NimbleParsec](https://github.com/dashbitco/nimble_parsec) and [ExSpirit](https://github.com/OvermindDL1/ex_spirit) so why write another one?\n\nFirst I am learning about writing parsers with parser combinators and implementing my own seemed like a good way to learn more. I was especially motivated by Saša Jurić's talk where he [builds up parser combinators from the ground up](https://www.youtube.com/watch?v=xNzoerDljjo). Ergo owes a lot to his style.\n\nSecond the code of both ExSpirit & NimbleParsec is, given my relatively limited experience of Elixir, a little hard to digest. I wanted to handle errors better than I'd seen in any of their examples (which seemed to ignore this issue) and finding the code hard to understand made it difficult to do that.\n\nSo there are some notable differences between Ergo and NP/ExS that perhaps justify its existence:\n\nFirst, and maybe difficult to spot, is that Ergo is implemented almost exclusively using functions and not macros while ExS/NP both make significant use of macros. My understanding is not sophisticated enough to understand quite what the macros are buying the user of those libraries but I was able to do without them except in one case. I introduced the\n`lazy()` parser combinator to handle parser recursion (I came across this when parsing values which may also be lists\nof values).\n\nSecond Ergo is built with error handling as one of its priorities. As a newbie I felt the need, so internally, Ergo parsers are not bare functions but `Parser` structs that assist with debugging. This takes the form of cycle detection\nand telemetry. See [debugging](debugging.md) for more information.\n\nThird, ExS/NP make heavy use of the Elixir `|>` operator to combine parsers together, for example:\n\n```elixir\nact =\n    string(\"Act\")\n    |> replace(:act)\n    |> ignore(whitespace)\n    |> concat(id)\n    |> ignore(whitespace)\n    |> ignore(char(?{))\n    |> ignore(whitespace)\n    |> optional(attributes)\n    |> ignore(whitespace)\n    |> concat(scene)\n    |> repeat(ignore(whitespace) |> concat(scene))\n    |> ignore(whitespace)\n    |> ignore(char(?}))\n    |> wrap\n```\n\nwhile in Ergo style you'd write:\n\n```elixir\nact =\n    sequence([\n        literal(\"Act\", ast: fn _ast -> :act end),\n        ignore(whitespace),\n        id,\n        ignore(whitespace),\n        ignore(char(?{)),\n        ignore(whitespace),\n        optional(attribtues),\n        ignore(whitespace),\n        many(scene),\n        ignore(whitespace),\n        ignore(char(?}))\n    ])\n```\n\nPerhaps unsurprisingly I prefer the Ergo style.\n\nIf you need something robust and high-performance I suspect you should be using NimbleParsec rather than Ergo. NP is written by Jose Valim himself and my understanding is that it makes good use of Elixir binary handling for performance.\n\nIf, on the other hand, you prefer the Ergo style and would benefit from the debugging support, you could try Ergo and I'd love to hear from you if you do.","ref":"comparisons.html","title":"Comparisons with other libraries","type":"extras"},{"doc":"# Creating a Basic Parser\n\nThis tutorial will explore how Ergo works through creating a parser to match numbers, be they integers like -5 or 42, or decimals like 25.6. The parser will match them and convert them into the appropriate Elixir numeric value. We will build it up in stages, starting with bare digits.","ref":"basic_parser.html","title":"Creating a Basic Parser","type":"extras"},{"doc":"To begin with we need to be able to parse digits. One option for that is to use the basic `char` parser to match digit characters. Using IEx here is what you would do:\n\n```elixir\nalias Ergo\nalias Ergo.Context\nimport Ergo.Terminals\n\ndigit = char(?0..?9)\nErgo.parse(digit, \"42\")\n%Context{status: :ok, ast: 52}\n```\n\nWhere the integer 52 is the character code of the digit '4' (Try typing `?4` into the IEx console to see for yourself).\n\nNow we can parse a single digit, how about multiple digits?","ref":"basic_parser.html#parsing-a-digit","title":"Parsing a digit - Creating a Basic Parser","type":"extras"},{"doc":"To parse multiple digits we use the `many` parser in conjunction with the `digit` parser as follows:\n\n```elixir\nimport Ergo.Combinators\n\ndigits = many(digit())\nErgo.parse(digits, \"42\")\n%Context{status: :ok, ast: [52, 50]}\n```\n\nIn fact you might see `'42'` in your version of the AST because IEx will try to render the list `[52, 50]` as a charlist. This is a hangover from Erlang. If you would prefer to see the list add the following to your `~/.iex.exs` file:\n\n```elixir\nIEx.configure(inspect: [charlists: :as_lists])\n```\n\nThe `many` combinator parser repeatedly invokes the `digit` parser to match as many digit characters as possible, generating an AST that is a list of those digits. To transform them into a numeric value we need to apply another function to the AST.\n\nIn our case the AST list contains character values of the digits matched, e.g. [52, 50] for the digits ['4', '2'] respectively. Since the digit character '0' has character value of 48 we can turn the characters into digit values by subtracting 48.\n\n The heavy lifting of transforming digits is done by `c_transform` below which is pipeline to transform ['4', '2'] -> [{4, 10}, {2, 1}] -> [40, 2] -> 42:\n\n```elixir\nc_transform = fn ast ->\n  bases = Stream.unfold(1, fn n -> {n, n * 10} end)\n  digits = Enum.map(ast, fn digit -> digit - 48 end)\n  Enum.zip(Enum.reverse(digits), bases)\n  |> Enum.map(&Tuple.product/1)\n  |> Enum.sum\n  end\n\ndigits = many(digit) |> transform(c_transform)\n\nErgo.parse(digits, \"42\")\n%Context{status: :ok, ast: 42}\n```\n\nIn this case we are applying the `transform` parser to the `many` parser. Transform only operates on the AST of the parser it is given by applying a function to it. In this case we could also have used:\n\n```elixir\ndigits = many(digit(), ast: c_transform)\n```\n\nAs many of the combinator parsers support an optional `ctx:` or `ast:` argument as a shortcut.\n\nAt this point we can parse positive integers of any length:\n\n```elixir\nErgo.parse(digits, \"918212812783918723\")\n%Context{status: :ok, ast: 918212812783918723}\n```","ref":"basic_parser.html#parsing-many-digits","title":"Parsing many digits - Creating a Basic Parser","type":"extras"},{"doc":"What about negative values? We need to look for a leading '-' character however, unlike the digits, the minus is optional. Parsing the minus is simple enough:\n\n```elixir\nminus = char(?-)\n\nErgo.parse(minus, \"-\")\n%Context{status: :ok, ast: 45}\n```\n\n45 is the char value of the char '-'. We can now use the `optional` combinator to allow a minus to be matched, or not:\n\n```elixir\nminus = optional(char(?-))\n\nErgo.parse(minus, \"-42\")\n%Context{status: :ok, ast: 45}\n\nErgo.parse(minus, \"42\")\n%Context{status: :ok, ast: nil}\n```\n\nIn the second case the status is `:ok` meaning the optional parser succeeded, however the ast is `nil` meaning nothing was matched. Let's make this a bit more useful:\n\n```elixir\nminus = optional(char(?-)) |> transform(fn ast ->\n  case ast do\n    nil -> 1\n    45 -> -1\n  end\nend)\n\nErgo.parse(minus, \"-42\")\n%Context{status: :ok, ast: -1}\n\nErgo.parse(minus, \"42\")\n%Context{status: :ok, ast: 1}\n\n```\n\nNow when `minus` matches a '-' it will transform it to the value -1. When it doesn't match anything it will transform it to the value 1. Now let's combine it with the other parser.\n\n```elixir\ninteger = sequence([\n  minus,\n  digits\n])\n```\n\nThe `sequence` parser tries to match a list of parser in turn and, if they all match, generates an AST composed of a list of each of their results. Let's see how it works:\n\n```elixir\nErgo.parse(integer, \"1234\")\n%Context{status: :ok, ast: [1, 1234]}\n\nErgo.parse(integer, \"-5678\")\n%Context{status: :ok, ast: [-1, 5678]}\n```\n\nSo we can see that it's easy to get the right result by simply taking the product of the two values in the AST:\n\n```elixir\ninteger = sequence([\n  minus,\n  digits,\n  ],\n  ast: &Enum.product/1\n)\n\nErgo.parse(integer, \"1234\")\n%Context{status: :ok, ast: 1234}\n\nErgo.parse(integer, \"-5678\")\n%Context{status: :ok, ast: -5678}\n```\n\nSo far so good. We can now parse positive and negative integers.","ref":"basic_parser.html#parsing-negative-numbers","title":"Parsing negative numbers - Creating a Basic Parser","type":"extras"},{"doc":"If we want to parse decimal numbers as well we need to handle the (optional) mantissa, the digits to the right of the decimal point.\n\nWe can see that the mantissa is structurally the same as the integer part, a set of digits, but will need to be processed a little differently.\n\nThe `m_transform` function below should look familiar. It works the same way as the `c_transform` only instead of multiplying by increasing powers of 10, we're dividing by increasing powers of 10.\n\n```elixir\nm_transform = fn ast ->\n  ast\n  |> Enum.map(fn digit -> digit - 48 end)\n  |> Enum.zip(Stream.unfold(0.1, fn n -> {n, n / 10} end))\n  |> Enum.map(&Tuple.product/1)\n  |> Enum.sum\nend\n\nmantissa = many(digit, ast: m_transform)\n\nErgo.parse(mantissa, \"5\")\n%Context{status: :ok, ast: 0.5}\n\nErgo.parse(mantissa, \"42\")\n%Context{status: :ok, ast: 0.42000000000000004}\n```\n\nThere may be a precision issue with this code but you can see the principle it is operating by.\n\nNow to join the two components together, assuming there is a decimal point (suggesting we'll need `optional` again). Also we'll again make use of the `ast:` feature of the `sequence` combinator to process AST's to give us the right value.\n\n```elixir\nnumber = sequence([\n  integer,\n  optional(\n    sequence([\n      ignore(char(?.)),\n      mantissa\n    ], ast: &List.first/1)\n  )\n], ast: &Enum.sum/1)\n\nErgo.parse(number, \"42\")\n%Context{status: :ok, ast: 42}\n\nErgo.parse(number, \"0.45\")\n%Context{status: :ok, ast: 0.45}\n\nErgo.parse(number, \"-42\")\n%Context{status: :ok, ast: -42}\n```\n\nAll looking good, just one more example:\n\n```elixir\nErgo.parse(number, \"-4.2\")\n%Context{status: :ok, ast: -3.8}\n```\n\nOops! There is a problem with our implementation in that we add together the integer and decimal parts. This works for positive numbers but in the latter case -4 + 0.2 = -3.8 not -4.2. When the integer part is negative we need to subtract the decimal part. We can no longer just use `Enum.sum` to process the result of the top-level sequence. Instead:\n\n```elixir\ncombine = fn\n  [integer, decimal | []] ->\n    if integer >= 0 do\n      integer + decimal\n    else\n      integer - decimal\n    end\n  ast ->\n    Enum.sum(ast)\nend\n\nnumber = sequence([\n  integer,\n  optional(\n    sequence([\n      ignore(char(?.)),\n      mantissa\n    ], ast: &List.first/1)\n  )\n], ast: combine)\n\nErgo.parse(number, \"-4.2\")\n%Context{status: :ok, ast: -4.2}\n```","ref":"basic_parser.html#parsing-decimals","title":"Parsing decimals - Creating a Basic Parser","type":"extras"},{"doc":"Through a series of steps we have built a parser that can handle any kind of integer or decimal number we throw at it. We've seen the using of terminal parsers like `char` as well as combinator parsers like `optional`, `ignore`, `many`, and `sequence` and meta parsers like `transform` (and that often transform can be specified as a `ast:` argument to a combinator parser).\n\nHopefully this guide will be helpful in thinking about how to build your own parsers.","ref":"basic_parser.html#conclusion","title":"Conclusion - Creating a Basic Parser","type":"extras"},{"doc":"# Debugging\n\nAs parsers become more complex it can be difficult to work out why they fail to operate as expected for a given input,\neven a stream of events can be difficult to work with. Ergo parses emit telemetry as they run that is then threaded into\nan outline. In this was the debugging information is nested as per the parsers themselves and uninteresting information\nis easily hidden to aid focusing on the output that matters.\n\nThe telemetry is implemented using the [Telemetry](https://hex.pm/packages/telemetry) package which is quite commonly\nused for this purpose by other library's (e.g. Ecto). By default telemetry events are ignored. In order to save them\nstart the Telemetry service and get the events once parsing is complete.\n\n```\nErgo.Telemetry.start()\n%{status: :ok, id: id} = Ergo.parse(…)\nErgo.Telemetry.get_events(id)\n```\n\nThe events can be inspected as a list. However it may be more useful to convert into an outline, by default an OPML\ndocument that can be loaded into most outliners.\n\n```\nFile.write(\"debugging.opml\", Ergo.Outline.OPML.generate_opml(\"My ID\", events))\n```\n\n# Cycles\n\nA challenge when building parsers is accidentally creating a cycle where the parser will never finish but loop over the same input forever. For example:\n\n```elixir\nmany(choice([many(ws()), char(?})]))\n```\n\nGiven an input like \"}}}\" will never finish. The inner many clause will always succeed with 0 whitespace characters and never actually process the `char` parser at all. You wouldn't deliberately set out to write such a parser but it can happen.\n\nFor this reason Ergo implements cycle detection. If the same parser is run a second time on the same input we know we have hit a cycle and an `Ergo.Context.CycleError` will be raised.","ref":"debugging.html","title":"Debugging","type":"extras"},{"doc":"# Recursion","ref":"recursion.html","title":"Recursion","type":"extras"},{"doc":"Parser combinators, being a form of recursive descent parser, are unable to\nhandle left recursion.\n\nIn general this looks like:\n\nA → A𝛼 | β\n\nWhere non-terminal A ends up being substituted for itself before any token\ncan be matched, leading to A → A𝛼 | β again, and so on and so on in an\ninfinite recursion.\n\nSuch parsers can usually be rewritten to a form that is not left-recursive\nhowever that is beyond the scope of this guide.","ref":"recursion.html#left-recursion","title":"Left Recursion - Recursion","type":"extras"},{"doc":"There is a further problem that arises from the way parser combinators are\ndefined as functions returning in a language that is not, by default, lazy.\nHere is an example parser that is designed to parse values like 42, true,\n\"What is six times seven?\" and also lists of values, including other lists.\nNote that we elide handling white space for brevity.\n\n```elixir\ndef value() do\n  choice([\n    number_val(),\n    string_val(),\n    boolean_val(),\n    list_val()\n  ])\nend\n\ndef list_val() do\n  sequence([\n    char(?[)\n    value(),\n    many(\n      sequence([\n        comma(),\n        value()\n      ])\n    ),\n    char(?])\n  ])\nend\n```\n\nA value can be a number, string, boolean, or list. But list is a sequence of\nvalues. The problem arises from a call to value() leading to a call to list()\nwhich, in turn, leads to a call to value(), leading to a call to list() and\nso on in an infinite recursion.\n\nNote that this is not grammatical recursion as in the left recursion example\nabove and cannot be solved by rewriting. The issue is that Elixir evaluates\nfunction calls eagerly, i.e. when they are encountered. We need to introduce\na \"call gap\" to break the recursion.\n\nWe do this with the `lazy` parser combinator. Lazy wraps it's parser in a\nfunction so that it is not called immediately, breaking the recursion. E.g.:\n\n```elixir\ndef value() do\n  choice([\n    number_val(),\n    string_val(),\n    boolean_val(),\n    lazy(list_val())\n  ])\nend\n```","ref":"recursion.html#eager-recursion","title":"Eager Recursion - Recursion","type":"extras"},{"doc":"# General Guidance","ref":"guidance.html","title":"General Guidance","type":"extras"},{"doc":"In my experience when you already have an idea of what it is you want to parse there is a temptation to start at the top and work down. Whenever I do this I seem to run into trouble with something that doesn't fit together right or parsing issues that I then find hard to track down.\n\nMy experience is that it's easiest to work bottom up. Parse small actual pieces of the language you want to parse and join these together, building towards the whole format. In general I tend to find I hit parsing errors quicker and can resolve them more easily this way.\n\nWhile you might have a different experience my advice, if you don't have an opinion yet, is to build your parser bottom up.","ref":"guidance.html#top-down-or-bottom-up","title":"Top-down or Bottom-up? - General Guidance","type":"extras"}],"producer":{"name":"ex_doc","version":"0.36.1"},"proglang":"elixir"}